/*full splitting
  Example illustrating the use of the BlockIP library for stochastics problems.
  The following two-stage problem consists of determining the optimal
capacity investment in various types of power plants so as to meet
next period demands for electricity. FThe number of power plants an modes 
can change. The next period demand for each of the three modes are to be met. 
There is a budget constraint and also a constraint on the minimum total capacity.
The problem is written in Primal form with splitting (for solve with BlockIP)
 */

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cstring>
#include <vector>
#include "math.h"
#include "BlockIP.h"
#include "ExceptionBlockIP.h"

 int rest1stage=0;//number of constraints first stage
 int techno = 0; //number of available technologies (i) //4
 int modesEd = 0; //number of modes of electicity demand (j) //3
 int Blocks = 0;  // number of blocks (scenarios 1ºstage+2ºscenarios)//3

using namespace std;
void write_options();
void read_options_Electric(int argc, char *argv[], int &index_outfile, int &index_mpsfile, bool &onlymps, bool &quad, int &rest1stage, int &techno, int &modesEd, int &Blocks);
void read_options_BlockIP(BlockIP *bip, int argc, char *argv[]);
//prototipos
void crearN1_MINCAP(int techno, int *iN1, int *jN1, int *aN1);
void crearN1_BUDGET(int techno, int *iN1, int *jN1, int *aN1);
void crearN1_VarX(int techno,int rest1stage, int *iN1, int *jN1, int *aN1);
void crearN1_VarYa(int techno,int modesEd, int rest1stage, int *iN1, int *jN1, int *aN1);
void crearN1_VarYb(int techno,int modesEd, int rest1stage, int *iN1, int *jN1, int *aN1);
void crearN1_holguras(int techno,int modesEd, int rest1stage, int *iN1, int *jN1, int *aN1);
void crearN2_VarX(int techno, int *iN2, int *jN2, int *aN2);
void crearN2_VarYa(int techno,int modesEd, int *iN2, int *jN2, int *aN2);
void crearN2_VarYb(int techno,int modesEd, int *iN2, int *jN2, int *aN2);
void crearN2_holguras(int techno,int modesEd, int *iN2, int *jN2, int *aN2);

void createInvestment(int techno,double *c1); 
void createProductionCost(int techno, int modesEd, double *c2);

void createCost(int techno, int modesEd, int Blocks, int VarN1, int VarN2,int VarBlocks, double *c1, double *c2, double *c);
void createDemand(int techno, int modesEd, int Blocks, int rest1stage, int restN1, int restN2, int rest, int Lslacks, double *dema);

int main(int argc, char *argv[]) {
  clock_t t0,t1;

  if (argc < 6 ){   //|| argc % 2 != 0) {
    cerr << "usage: Electric need the <number of Supplies>  <number Pfailities> <number of clients> and <number of blocks>  "<< endl
         << "example: ./Electric -ElectricEsc #Supply #pFacilities #Clients #scenarios -type_comp_dy 0 \n"
         << endl;

    write_options();
    exit(-1);
  }

 // read some options from command line
 int rest1stage=0;//number of constraints first stage
 int techno = 0; //number of available technologies (i) //4
 int modesEd = 0; //number of modes of electicity demand (j) //3
 int Blocks = 0;  // number of blocks (scenarios 1ºstage+2ºscenarios)//3

// for read options Electric
  int index_outfile= -1;
  int index_mpsfile= -1;
  bool onlymps= false;
  bool quad= false;
 
  read_options_Electric(argc,argv,index_outfile,index_mpsfile,onlymps,quad,rest1stage, techno, modesEd, Blocks);
 //for(int i = 0; i < argc; i++) 
  //   cout << argv[i] << " "; 
  // cout << endl; 
/*
 int rest1stage=2;//number of constraints first stage
 int techno = 4; //number of available technologies (i) //4
 int modesEd = 3; //number of modes of electicity demand (j) //3
 int Blocks = 3;  // number of blocks (scenarios 1ºstage+2ºscenarios)//3 
*/
 int linking = 0; // if 0 then linking are of type SPLITTING; if 1 then linking are GENERAL matrices 
 int nzN1 = techno*rest1stage+techno+(techno*modesEd)+(techno*modesEd)+rest1stage+techno;
 int nzN2 = techno+(techno*modesEd)+(techno*modesEd)+techno;
 int VarY = techno*modesEd; //number of variables 2º stage (1 block)
 int restN1=rest1stage+techno+modesEd;
 int VarN1=techno+(techno*modesEd)+techno+rest1stage;
 int restN2=techno+modesEd;
 int VarN2=techno+(techno*modesEd)+techno;
 int VarBlocks=VarN1+(VarN2*(Blocks-1));
 int rest=restN1+(restN2*(Blocks-1));
 int Lslacks = (techno)*(Blocks-1); 

cout<<"INICIA Electric"<<endl;

 int *aN1=NULL, *jN1=NULL, *iN1=NULL;
 int *aN2=NULL, *jN2=NULL, *iN2=NULL;
 double *c1=NULL, *c2=NULL, *c=NULL, *dema=NULL;

  srand(50); 

cout<<"Blocks= "<<Blocks<<endl;
cout<<"techno= "<<techno<<endl;
cout<<"modesEd= "<<modesEd<<endl;
cout<<"VarBlocks= "<<VarBlocks<<endl;
cout<<"1stStage= "<<techno<<endl;


  iN1=new int[nzN1];
  jN1=new int[nzN1];
  aN1=new int[nzN1];
  iN2=new int[nzN2];
  jN2=new int[nzN2];
  aN2=new int[nzN2];
  c1=new double[techno];
  c2=new double[VarY];
  c=new double[VarBlocks];
  dema=new double[rest+Lslacks];

 crearN1_MINCAP(techno, iN1, jN1,aN1);
 crearN1_BUDGET(techno, iN1, jN1,aN1);
 crearN1_VarX(techno,rest1stage, iN1, jN1,aN1);
 crearN1_VarYa(techno,modesEd, rest1stage, iN1, jN1,aN1);
 crearN1_VarYb(techno,modesEd, rest1stage, iN1, jN1,aN1);
 crearN1_holguras(techno, modesEd, rest1stage, iN1, jN1, aN1);
 crearN2_VarX(techno, iN2, jN2,aN2);
 crearN2_VarYa(techno,modesEd, iN2, jN2, aN2);
 crearN2_VarYb(techno,modesEd, iN2, jN2, aN2);
 crearN2_holguras(techno, modesEd, iN2, jN2, aN2);

 createInvestment(techno, c1);
 createProductionCost(techno, modesEd, c2); 
 createCost(techno, modesEd, Blocks,VarN1, VarN2,VarBlocks, c1, c2, c);
 createDemand(techno, modesEd, Blocks, rest1stage, restN1, restN2, rest, Lslacks, dema);
//for check N1
//for(int i=0; i<nzN1;i++){
//	cout<<"k= "<<i<<" i= "<<iN1[i]<<" j= "<<jN1[i]<<" a="<<aN1[i]<<endl;
//}

//for check N2
//for(int i=0; i<nzN2;i++){
//	cout<<"k= "<<i<<" i= "<<iN2[i]<<" j= "<<jN2[i]<<" a="<<aN2[i]<<endl;
//}


  MatrixBlockIP *N =  NULL; 
  N = new MatrixBlockIP[Blocks];

  int *iW, *jW;
  double *WN; 
  int nz_aux;


for(int b=0; b<Blocks; b++){ 
	if(b==0){//first block first stage and first scenario
		N[b].create_general_matrix_format_ija(restN1, VarN1, nzN1, iW, jW, WN); 
		
		for (int i = 0; i < nzN1; ++i) {
			iW[i] = iN1[i];
			jW[i] = jN1[i]; 
			WN[i] = aN1[i];
			//cout<<"i1= "<<iW[i]<<"  j= "<<jW[i]<<" = Wa["<<i<<"]= "<<WN[i]<<endl;
		}//for
	}else{
		N[b].create_general_matrix_format_ija(restN2, VarN2, nzN2, iW, jW, WN); 
		for (int i = 0; i < nzN2; ++i) {
			iW[i] = iN2[i];
			jW[i] = jN2[i]; 
			WN[i] = aN2[i];
			//cout<<"i2= "<<iW[i]<<"  j= "<<jW[i]<<" = Wa["<<i<<"]= "<<WN[i]<<endl;
		}//for

	}//else
}//forb

//for check N[0]
/*
cout<<"N[0].nz= "<<N[0].nz<<endl;
for(int i=0; i<N[0].nz;i++){
	//cout<<"k= "<<i<<" i= "<<iN2[i]<<" j= "<<jN2[i]<<" a="<<aN2[i]<<endl; N.a[i]
	cout<<" N[0].a["<<i<<"]="<< N[0].a[i]<<endl;
}
*/
//for check N[2]
/*
cout<<"N[2].nz= "<<N[2].nz<<endl;
for(int i=0; i<N[2].nz;i++){
	//cout<<"k= "<<i<<" i= "<<iN2[i]<<" j= "<<jN2[i]<<" a="<<aN2[i]<<endl; N.a[i]
	cout<<" N[2].a["<<i<<"]="<< N[2].a[i]<<endl;
}
*/

  MatrixBlockIP *L = NULL;
  //create the linking matrices depending on the model chosen and value of parameter linking
  //MatrixBlockIP *L = new MatrixBlockIP[(linking==0) ? 1 : numBlocks];		
  L = new MatrixBlockIP[(linking==0) ? 1 : Blocks];	

 if(linking==0){
	cout<<"Splitting format"<<endl;
    // linking of type splitting: blocks, range of splitted variables is 0:numVary+numVarU-1
      L->create_splitting_matrix(Blocks,0,(techno-1)); //***numVarU
  }
  else {
  	cout<<"No splitting format"<<endl;
}//else


 BlockIP bip; // declare BlockIP problem
    
   double *cost=NULL, *qcost=NULL, *ub=NULL, *rhs=NULL;
  
 // creation of BlockIP problem
  bip.create_problem(BlockIP::QUADRATIC, cost, qcost, NULL, NULL, ub, rhs, Blocks, false, N, (linking==0), L);//&N(linking==0)


 // fill arrays of data for problem -cost 
	for (int k=0; k<VarBlocks; k++){	
		if(k<techno){   
			cost[k] = c[k];
		}else{
				cost[k] = (c[k])*(1.0/(double)Blocks);
		}	
	} //for  
	for (int i = VarBlocks; i <VarBlocks+Lslacks; ++i) {
	   	cost[i] = 0;  
	}//for
  //for check
	//for(int i = 0; i < VarBlocks+Lslacks; i++){
	//	cout<< "cost["<<i<<"]= "<<cost[i]<<endl;
	//}


 // fill arrays of data for problem -qcost 
	for (int k=0; k<VarBlocks; k++){	
		if(k<techno){   
			qcost[k] = 2;
		}else{
			if(cost[k]==0){
				qcost[k] = 0;
			}else{
				qcost[k] =2*(1.0/(double)Blocks);
			}//else
		}//else	
	} //for  
	for (int i = VarBlocks; i <VarBlocks+Lslacks; ++i) {
	   	qcost[i] = 0;  
	}//for
  //for check
	//for(int i = 0; i < VarBlocks; i++){
	//	cout<< "qcost["<<i<<"]= "<<qcost[i]<<endl;
	//}


 // rhs of constraints 
	for (int k=0; k<rest+Lslacks; k++){  
			rhs[k] = dema[k];
	}//for
  //for check
   //for(int k=0; k<rest+Lslacks;k++){    
   //  cout<<"rhs["<<k<<"]= "<<rhs[k]<<endl;
   //}

 // upper bound for each variable 
	for (int k=0; k<VarBlocks; k++){	
		ub[k] =1.0e+11;//para pequeños 1.0e+6;
		//ub[k] = bip.get_inf();
	}//for 
	for (int i = VarBlocks; i <VarBlocks+Lslacks; ++i) {
	   	ub[i] = 1.0e-8;  
	}//for
  //for check
   //for(int k=0; k<VarBlocks+Lslacks;k++){    
   //   cout<<"ub["<<k<<"]= "<<ub[k]<<endl;
   //}

  // change default parameters according to command line values given by user
  read_options_BlockIP(&bip, argc, argv);


//  cout << endl << "Write block-MPS format" << endl;
  bip.write_mps("ElPQuad");
 // set some options
  bip.set_show_specrad(true); // compute and show estimation of spectral radius

  cout << "BIPnumConst = " << bip.get_km() << endl; //Get the number of constraints without linking constraints
  cout << "BIPnumVaraibles = " << bip.get_kn() << endl;//Get the number of variables without linking constraints
  cout << "BIPnumLinking = " << bip.get_l_link() << endl; //Get the number of linking constraints.
  cout << "numConstOrig = " << bip.get_num_cons() << endl; //Get the number of constraints of the original 
  cout << "numVarOrig = " << bip.get_num_vars() << endl; //Get the number of variables of the original
cout << " optim_gap= " << bip.get_optim_gap() << endl; //Get the optimality gap de BlockIP 
 cout << "init_pcgtol= " << bip.get_init_pcgtol() << endl; //Get  tolerance of the pcg (default in linear problems 1.e-2)
 cout << "red_pcgtol= " << bip.get_red_pcgtol() << endl; //Get if is 1 the tol of pcg don't change in every iteration, number between [0,1] a cada iteración 
cout << "m_pw_prec= " << bip.get_m_pw_prec() << endl;

//solve

  cout << "Solve Problem ** "<<endl; 
 t0= clock();
  bip.minimize(); // solve problem
 t1= clock();

#ifdef OPENMP
    cout << "Elapsed OpenMP wall clock time: " << bip.get_wclock_time() << endl;
#endif
  cout << "Optimization CPU time: " << (double) (t1-t0)/CLOCKS_PER_SEC << " seconds" << endl;
  cout << "Total IPit= " << bip.get_it() << endl;
  cout << "Total CGit= " << bip.get_cgit() << endl;
  cout << "Primal Objetive = " << bip.get_fobj() << endl;
  cout << "Dual objetive = " << bip.get_dobj() << endl;
////////////////////////
delete[] iN1;
delete[] jN1;
delete[] aN1;
delete[] iN2;
delete[] jN2;
delete[] aN2;
delete[] c1;
delete[] c2;
delete[] c;
delete[] dema;


delete [] N;
delete [] L;

  return 0;
}//main
void createDemand(int techno, int modesEd, int Blocks, int rest1stage, int restN1, int restN2, int rest, int Lslacks, double *dema){
    int rr=0; 
	for(int k=0; k<rest;k++){
		if(k<rest1stage){
			if(k==0){ //first constraint MINCAP
				dema[k]=3*techno;
				//cout<<"dema["<<k<<"]= "<<dema[k]<<endl;
			}else{//if
					dema[k]=3000*techno;
					//cout<<"dema["<<k<<"]= "<<dema[k]<<endl;					
				}//else k=1
			}else{//inicia rest for Blocks	
				if(k<techno+rest1stage+(restN2*rr)){ //4
					dema[k]=0;
					//cout<<"dema["<<k<<"]= "<<dema[k]<<endl;
				}else{////modosEd demanda
					dema[k]=(rand() % 10 + 2);
					//cout<<"dema["<<k<<"]= "<<dema[k]<<endl;
				}//else
				if(k==rest1stage+(restN2*(rr+1)-1)){
				rr++;
				//auxk1=0;
				}//if
			}//else Blocks
	}//for rest

	for(int k=0; k<Lslacks;k++){ 
		int auxk=rest; 
		dema[auxk+k]=0;  
		//cout<<"dema["<<k+auxk<<"]= "<<dema[auxk+k]<<endl;
	}

}//void

void createCost(int techno, int modesEd, int Blocks, int VarN1, int VarN2, int VarBlocks, double *c1, double *c2,double *c){

int auxk=0, auxk1=0, rr=0; 
	for(int k=0; k<VarBlocks;k++){
		if(k<VarN1){
			if(k<techno){ //4
				c[k]=c1[k];
				//cout<<"c["<<k<<"]= "<<c[k]<<endl;
			}else{//if
				if(k<techno+(techno*modesEd)){//16
					c[k]=c2[auxk1];					
					auxk1++;
					//cout<<"c["<<k<<"]= "<<c[k]<<endl;
				}else{//if
					c[k]=0;
					//cout<<"c["<<k<<"]= "<<c[k]<<endl;
					}//else
				}//else
			if(k==VarN1-1){
				auxk1=0;
			}//if 
		}else{//forVarN1 inicia N2's	
			if(k<techno+VarN1+(VarN2*rr)){ //4
				c[k]=0;
				//cout<<"c["<<k<<"]= "<<c[k]<<endl;
				}else{//if 4
					if(k<techno+(techno*modesEd)+VarN1+(VarN2*rr)){
						c[k]=c2[auxk1];
						auxk1++;
						//cout<<"c["<<k<<"]= "<<c[k]<<endl;
					}else{//if
						c[k]=0;
						//cout<<"c["<<k<<"]= "<<c[k]<<endl;
						}//else
					}//else
				if(k==VarN1+(VarN2*(rr+1)-1)){
				rr++;
				auxk1=0;
				}//if
			}//else N2's
	}//forVarBlocks
	//for(int k=0; k<VarBlocks;k++){    
	//	cout<<"c["<<k<<"]= "<<c[k]<<endl;
	//}
}//void

void createInvestment(int techno, double *c1){ 
	for(int k=0; k<techno;k++){ 
		c1[k]=(rand() % 20 + 5); 
		//cout<<"c1["<<k<<"]= "<<c1[k]<<endl;
	}//for
}//void

void createProductionCost(int techno, int modesEd, double *c2){ 
   double *auxt=NULL, *auxmEd=NULL;
   int auxk=0;
    auxt=new double[techno];
	auxmEd=new double[modesEd]; 

	for(int k=0; k<techno;k++){ 
		auxt[k]=(rand() % (500-200)) / 100.0;
		//cout<<"auxt["<<k<<"]= "<<auxt[k]<<endl;
	}

	for(int k=0; k<modesEd;k++){ 
		auxmEd[k]=(rand() % 8 + 0);
		//cout<<"auxmEd["<<k<<"]= "<<auxmEd[k]<<endl;
	}
	
	for(int i=0; i<modesEd;i++){ 
		for(int j=0; j<techno;j++){
		c2[auxk]=auxt[j]*auxmEd[i];
		//cout<<"c2["<<auxk<<"]= "<<c2[auxk]<<endl;
		auxk++;
		}//for
	}//for

	//for(int k=0; k<techno*modesEd;k++){ 
		//cout<<"c2a["<<k<<"]= "<<c2[k]<<endl;
	//}

	delete[] auxt;
	delete[] auxmEd;
}//void

void crearN1_MINCAP(int techno, int *iN1, int *jN1, int *aN1){
  int aux1i=0, aux1j=0;
	for(int k=0; k<techno;k++){ 
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN1[k]=aux1i;
		jN1[k]=aux1j;
		aN1[k]=1;
		aux1j++;
	}//for
}//void

void crearN1_BUDGET(int techno, int *iN1, int *jN1, int *aN1){
  int aux1i=1, aux1j=0;
	for(int k=techno; k<techno+techno;k++){ //8
		iN1[k]=aux1i;
		jN1[k]=aux1j;
		aN1[k]=(rand() % 100 + 5);
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" a = "<<aN1[k]<<" ->"<<k<<endl;
		aux1j++;
	}//for
}//void

void crearN1_VarX(int techno, int rest1stage, int *iN1, int *jN1, int *aN1){
  int aux1i=rest1stage, aux1j=0;
	for(int k=techno*rest1stage; k<(techno*rest1stage)+techno;k++){ //8
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN1[k]=aux1i;
		jN1[k]=aux1j;
		aN1[k]=-1;
		aux1j++;
		aux1i++;
	}//for
}//void

void crearN1_VarYa(int techno,int modesEd, int rest1stage, int *iN1, int *jN1, int *aN1){
  int rr=1, aux1i=rest1stage, aux1j=techno;
	for(int k=(techno*rest1stage)+techno; k<(techno*rest1stage)+techno+(techno*modesEd);k++){ //16
	//for(int k=0; k<(supplyN*Pfacility);k++){ //16
		//cout<<" ->"<<k<<" i= "<<aux1i<<" j= "<<aux1j<<endl;
		iN1[k]=aux1i;
		jN1[k]=aux1j;
		aN1[k]=1;
		aux1j++;
		if(k==((modesEd*rr)-1)+(techno*rest1stage)+techno){ //cada modesED
			//cout<<"cambio "<<((modesED*rr)-1) <<endl;//4 cambios=
			aux1i++;
			//aux1j=rr;
			rr++;
		}//if
	}//for
}//void

void crearN1_VarYb(int techno,int modesEd, int rest1stage, int *iN1, int *jN1, int *aN1){
  int rr=1, aux1i=rest1stage+techno, aux1j=techno, auxk= (techno*rest1stage)+techno+(techno*modesEd);
for(int k=auxk; k<auxk+(techno*modesEd);k++){ 	
	//for(int k=20; k<(supplyN*Pfacility);k++){ //16
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN1[k]=aux1i;
		jN1[k]=aux1j;
		aN1[k]=1;
		aux1j=aux1j+modesEd;
		if(k==((techno*rr)-1)+auxk){ //cada Pfacitity
			//cout<<"cambio " <<endl;//4 cambios=
			aux1i++;
			aux1j=techno+rr;
			rr++;
		}//if
	}//for
}//void

void crearN1_holguras(int techno,int modesEd, int rest1stage, int *iN1, int *jN1, int *aN1){
  int aux1i=0, aux1j=techno+(techno*modesEd), auxk=(techno*rest1stage)+techno+(techno*modesEd)+(techno*modesEd);
	for(int k=auxk; k<auxk+techno+rest1stage; k++){ //6
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN1[k]=aux1i;
		jN1[k]=aux1j;
		if(k==auxk){ 
			aN1[k]=-1;
		}else{
			aN1[k]=1;
		}
		aux1i++;
		aux1j++;
	}//for
}//void


void crearN2_VarX(int techno, int *iN2, int *jN2, int *aN2){
  int aux1i=0, aux1j=0;
	for(int k=0; k<techno;k++){ //8
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=-1;
		aux1j++;
		aux1i++;
	}//for
}//void


void crearN2_VarYa(int techno,int modesEd, int *iN2, int *jN2, int *aN2){
  int rr=1, aux1i=0, aux1j=techno;
	for(int k=techno; k<techno+(techno*modesEd);k++){ 
		//cout<<" ->"<<k<<" i= "<<aux1i<<" j= "<<aux1j<<endl;
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=1;
		aux1j++;
		if(k==((modesEd*rr)-1)+techno){ //cada modesED
			//cout<<"cambio "<<((modesEd*rr)-1) <<endl;//4 cambios=
			aux1i++;
			rr++;
		}//if
	}//for
}//void

void crearN2_VarYb(int techno,int modesEd, int *iN2, int *jN2, int *aN2){
  int rr=1, aux1i=techno, aux1j=techno, auxk=techno+(techno*modesEd);
for(int k=auxk; k<auxk+(techno*modesEd);k++){ 	
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=1;
		aux1j=aux1j+modesEd;
		if(k==((techno*rr)-1)+auxk){ //cada Pfacitity
			//cout<<"cambio " <<endl;//4 cambios=
			aux1i++;
			aux1j=techno+rr;
			rr++;
		}//if
	}//for
}//void

void crearN2_holguras(int techno,int modesEd, int *iN2, int *jN2, int *aN2){
  int aux1i=0, aux1j=techno+(techno*modesEd), auxk=techno+(techno*modesEd)+(techno*modesEd);
	for(int k=auxk; k<auxk+techno; k++){ //6
		//cout<<"i= "<<aux1i<<" j= "<<aux1j<<" ->"<<k<<endl;
		iN2[k]=aux1i;
		jN2[k]=aux1j;
		aN2[k]=1;
		aux1i++;
		aux1j++;
	}//for
}//void


/////////////////////////////////////////////////////////////////////////////////////////////////////////







void write_options() {
  cout <<
  "  -ElectricEsc {numberFirstConstraints} {numbertechno} {numbermodesEd} {Escenarios}(required) \n"
  "  -out {name}\n"
  "    Output file for optimal variables (if not provided, solution is not reported)\n"
  "  -mps {name}\n"
  "    If name is provided, an mps format of the problem is written to filename 'name'\n"
  "  -only_mps {name}\n"
  "    If name is provided, the program stops after  writing mps format (problem is not solved)\n"
  "  -quad {y/n}\n"
  "    If y costs are considered quadratic; otherwise linear (deefault is linear)\n"
  "  -inf {value}\n"
  "    Set infinity value to be considered\n"
  "  -m_pw_prec {value}\n"
  "    Set number of terms used as preconditioner of the power series expansion of (D-C'*B^{-1}*B)^{-1}\n"
  "  -sigma {value}\n"
  "    Set reduction of the centrality parameter at each IP iteration\n"
  "  -rho {value}\n"
  "    Set reduction of the step-length for the primal and dual variables at each IP iteration\n"
  "  -optim_gap {value}\n"
  "    Set optimality gap tolerance\n"
  "  -optim_pfeas {value}\n"
  "    Set primal feasibility tolerance\n"
  "  -optim_dfeas {value}\n"
  "    Set dual feasibility tolerance\n"
  "  -output_freq {value}\n"
  "    Set output information lines will be printed each output_freq IP iterations\n"
  "  -output {value}\n"
  "    Set type of output. Can take the values: 0= NONE, 1= SCREEN, 2= FILE, 3=BOTH\n"
  "  -maxiter {value}\n"
  "    Set maximum number of IP iterations\n"
  "  -min_pcgtol {value}\n"
  "    Set minimum tolerance for the conjugate gradient\n"
  "  -red_pcgtol {value}\n"
  "    Set reduction of pcg_tol at each IP iteration\n"
  "  -init_pcgtol {value}\n"
  "    Set initial tolerance for the conjugate gradient (by default PCG_TOL_LIN if linear problem, PCG_TOL_QUAD if quadratic or -convex- nonlinear)\n"
  "  -type_start_point {value}\n"
  "    Set how starting point is computed. Can take the values: 0= SIMPLE, 1= QUAD_EQCONS_PROB\n"
  "  -type_comp_dy {value}\n"
  "    Set how dy direction is computed. Can take the values: 0= CHOL_PWRS_PCG, 1= FULL_CHOL, 2= HYBRID_PCG, 3= CHOL_THETA0_PCG, 4= PWRS_THETA0_PCG, 5=THETA0_PWRS_PCG, 6=AUT_PWRSTHETA0_PCG, 7=DYN_PWRSTHETA0_PCG\n"
  "  -type_direction {value}\n"
  "    Set which direction is computed. Can take the values: 0= NEWTON, 1= SECOND_ORDER, 2= AUTOMATIC\n"
  "  -deactivateLnk {value}\n"
  "    Set flag on deactivation of linking\n"
  "  -type_reg {value}\n"
  "    Set type of regularization performed. Can take the values: 0=NO_REG, 1= QUAD_REG, 2= PROX_REG\n" 
  "  -factor_reg {value}\n"
  "    Set initial value for regularization term\n" 
  "  -show_specrad {y/n}\n"
  "    Whether estimation of spectral radius by Ritz values is shown at each IP iteration\n"
  "  -show_princ_angles {y/n}\n"
  "    Whether average principal angles between L and N are shown when computed\n"
  "  -threshold_angle {value}\n"
  "    Set threshold principal angle to switch between PWRS and THETA0 preconditioners\n"
  "  -threshold_specrad {value}\n"
  "    Set threshold spectral radius to switch between PWRS and THETA0 preconditioners\n"
  "  -it_Theta0PWRS {value}\n"
  "    Set number of iterations between checks to switch between PWRS and THETA0\n"
  "  -type_comp_angle {value}\n"
  "    Set how principal angles are computed. Possible values are: 0= LINEARCOMB, 1= AVERAGE\n"
  "  -gap_changeChol {value}\n"
  "    Change to Cholesky if duality gap below this value\n"
  "  -zero_pivots {value}\n"
  "    Max number of zero pivots allowed in Cholesky factorization\n"
  "  -show_zero_pivots {y/n}\n"
  "    Show_zero_pivots at each IP iteration\n"
  "  -stop_if_PCG_fails {y/n}\n"
  "    If PCG fails stop, do not switch to Cholesky, useful if instance is large\n"
  "  -freevars {0/1}\n"
  "    How to deal with free variables: 0=SPLIT , 1=REGULARIZE\n"
  "  -threads n \n"
  "    Set number of threads (by default 1; if n<=1, 1; otherwise min{n,max number of threads})\n"
  << endl;
}//end void write_options()

void read_options_Electric(int argc, char *argv[], int &index_outfile, int &index_mpsfile, bool &onlymps, bool &quad, int &rest1stage, int &techno, int &modesEd, int &Blocks) {
  char filename[512];
  char c;
//  int supplyNa, Pfacilitya, ClientNa, numBloksa;
  for (int argIndex = 1; argIndex < argc; argIndex += 2) {
    if (!strcmp(argv[argIndex], "-ElectricEsc") && sscanf(argv[argIndex+1], "%i", &rest1stage) && sscanf(argv[argIndex+2], "%i", &techno) &&  sscanf(argv[argIndex+3], "%i", &modesEd) && sscanf(argv[argIndex+4], "%i", &Blocks) ){
	argIndex +=3;
        index_outfile= argIndex+1;
	}//if !strcmp(argv[argIndex]
    else if (!strcmp(argv[argIndex], "-out") && sscanf(argv[argIndex+1], "%s", &filename) )
      index_outfile= argIndex+1;
    else if (!strcmp(argv[argIndex], "-mps") && sscanf(argv[argIndex+1], "%s", &filename) )
      index_mpsfile= argIndex+1;
    else if (!strcmp(argv[argIndex], "-only_mps") && sscanf(argv[argIndex+1], "%s", &filename) ) {
      index_mpsfile= argIndex+1;
      onlymps= true;
    }
    else if (!strcmp(argv[argIndex], "-quad") && sscanf(argv[argIndex+1], "%c", &c) ) {
      quad= (c== 'y');
    }
    else if (!strcmp(argv[argIndex], "-inf") ||
             !strcmp(argv[argIndex], "-m_pw_prec") ||
             !strcmp(argv[argIndex], "-sigma") ||
             !strcmp(argv[argIndex], "-rho") ||
             !strcmp(argv[argIndex], "-optim_gap") ||
             !strcmp(argv[argIndex], "-optim_pfeas") ||
             !strcmp(argv[argIndex], "-optim_dfeas") ||
             !strcmp(argv[argIndex], "-output_freq") ||
             !strcmp(argv[argIndex], "-output") ||
             !strcmp(argv[argIndex], "-maxiter") ||
             !strcmp(argv[argIndex], "-min_pcgtol") ||
             !strcmp(argv[argIndex], "-red_pcgtol") ||
             !strcmp(argv[argIndex], "-init_pcgtol") ||
             !strcmp(argv[argIndex], "-type_start_point") ||
             !strcmp(argv[argIndex], "-type_comp_dy") ||
             !strcmp(argv[argIndex], "-type_direction") ||
             !strcmp(argv[argIndex], "-deactivateLnk") ||
             !strcmp(argv[argIndex], "-type_reg") ||
             !strcmp(argv[argIndex], "-factor_reg") ||
             !strcmp(argv[argIndex], "-show_specrad") ||
             !strcmp(argv[argIndex], "-show_princ_angles") ||
             !strcmp(argv[argIndex], "-threshold_angle") ||
             !strcmp(argv[argIndex], "-threshold_specrad") ||
             !strcmp(argv[argIndex], "-it_Theta0PWRS") ||
             !strcmp(argv[argIndex], "-type_comp_angle") ||
             !strcmp(argv[argIndex], "-gap_changeChol") ||
             !strcmp(argv[argIndex], "-zero_pivots") ||
             !strcmp(argv[argIndex], "-show_zero_pivots") ||
             !strcmp(argv[argIndex], "-stop_if_PCG_fails") ||
		     !strcmp(argv[argIndex], "-freevars") ||
             !strcmp(argv[argIndex], "-threads") 
             )
      // do nothing 
      {}
    else {
      cerr << "usage: Electric  {options}\n";
      write_options();
      exit(-1);
    }
  }
}
 


void read_options_BlockIP(BlockIP *bip, int argc, char *argv[]) {
  double d;
  int i;
  char c;
  for (int argIndex = 6; argIndex < argc; argIndex += 2) {
    if (!strcmp(argv[argIndex], "-inf") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_inf(d);
    else if (!strcmp(argv[argIndex], "-m_pw_prec") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_m_pw_prec(i);
    else if (!strcmp(argv[argIndex], "-sigma") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_sigma(d);
    else if (!strcmp(argv[argIndex], "-rho") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_rho(d);
    else if (!strcmp(argv[argIndex], "-optim_gap") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_optim_gap(d);
    else if (!strcmp(argv[argIndex], "-optim_pfeas") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_optim_pfeas(d);
    else if (!strcmp(argv[argIndex], "-optim_dfeas") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_optim_dfeas(d);
    else if (!strcmp(argv[argIndex], "-output_freq") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_output_freq(i);
    else if (!strcmp(argv[argIndex], "-output") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_output((BlockIP::OUTPUT)i);
    else if (!strcmp(argv[argIndex], "-maxiter") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_maxiter(i);
    else if (!strcmp(argv[argIndex], "-min_pcgtol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_min_pcgtol(d);
    else if (!strcmp(argv[argIndex], "-red_pcgtol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_red_pcgtol(d);
    else if (!strcmp(argv[argIndex], "-init_pcgtol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_init_pcgtol(d);
    else if (!strcmp(argv[argIndex], "-type_start_point") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_start_point((BlockIP::TYPE_START_POINT)i);
    else if (!strcmp(argv[argIndex], "-type_comp_dy") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_comp_dy((BlockIP::TYPE_COMP_DY)i);
    else if (!strcmp(argv[argIndex], "-type_direction") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_direction((BlockIP::TYPE_DIRECTION)i);
    else if (!strcmp(argv[argIndex], "-deactivateLnk") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_deactivateLnk(i);
    else if (!strcmp(argv[argIndex], "-type_reg") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_reg((BlockIP::TYPE_REG)i);
    else if (!strcmp(argv[argIndex], "-factor_reg") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_factor_reg(d);
    else if (!strcmp(argv[argIndex], "-show_specrad") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_show_specrad(c=='y');
    else if (!strcmp(argv[argIndex], "-show_princ_angles") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_show_princ_angles(c=='y');
    else if (!strcmp(argv[argIndex], "-threshold_angle") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_threshold_angle(d);
    else if (!strcmp(argv[argIndex], "-threshold_specrad") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_threshold_specrad(d);
    else if (!strcmp(argv[argIndex], "-it_Theta0PWRS") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_it_Theta0PWRS(i);
    else if (!strcmp(argv[argIndex], "-type_comp_angle") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_comp_angle((BlockIP::TYPE_COMP_ANGLE)i);
    else if (!strcmp(argv[argIndex], "-gap_changeChol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_gap_change_to_Chol(d);
    else if (!strcmp(argv[argIndex], "-zero_pivots") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_allowed_zero_pivots(i);
    else if (!strcmp(argv[argIndex], "-show_zero_pivots") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_show_zero_pivots(c=='y');
    else if (!strcmp(argv[argIndex], "-stop_if_PCG_fails") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_stop_if_PCG_fails(c=='y');
    else if (!strcmp(argv[argIndex], "-freevars") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_deal_freevars((BlockIP::TYPE_DEAL_FREEVARS)i);
    else if (!strcmp(argv[argIndex], "-threads") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_num_threads(i);
    else if (!strcmp(argv[argIndex], "-ElectricEsc")||
	     !strcmp(argv[argIndex], "-out")||
	     !strcmp(argv[argIndex], "-mps")||
	     !strcmp(argv[argIndex], "-only_mps") ||
	     !strcmp(argv[argIndex], "-quad")
	     )
      // do nothing
      {}
    else {
      cerr << "usage: Bip_BlockIP {options}\n";
      write_options();
      exit(-1);
    }
  }
}//end read_options_BlockIP(BlockIP *bip, int argc, char *argv[])

