/*full splitting
  Example illustrating the use of the BlockIP library for stochastics problems.
  It formulates a small farmer problem,
  which is solved several times with different options.
 */

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <cstring>
#include <vector>
#include "math.h"
#include "BlockIP.h"
#include "ExceptionBlockIP.h"


using namespace std;
void write_options();
void read_options_SCPrimal(int argc, char *argv[], int &index_outfile, int &index_mpsfile, bool &onlymps, bool &quad, int &supplyN, int &Pfacility, int &ClientN, int &numBlocks);
void read_options_BlockIP(BlockIP *bip, int argc, char *argv[]);
//prototipos
void createBalancePfacility(int supplyN, int Pfacility, int ClientN, int *iN1, int *jN1, int *aN1, int numVarx);
void createDemandClientN(int supplyN, int Pfacility, int ClientN, int *iN1, int *jN1, int *aN1, int numVarx, int numVarzS1);
void createCapacitySPy(int *iN2y, int *jN2y, int *aN2y, int numVary);
void createCapacitySPxH(int supplyN, int Pfacility,int ClientN, int *iN2x, int *jN2x, int *aN2x, int numVarx, int nozeroN2x, int numVarzS1); // capacity with slacks 
void createInvestmentSP(double *c1, int numVary); //investment cost for building
void createShortageC(int ClientN, double *z1); //the unit shortage cost at each distribution center
void createTransportationSPyPC(int supplyN, int Pfacility, double *qd, int numVarx); //transportation cost from S to P & P to C
void createCost(int supplyN, int Pfacility, int ClientN, int numBlocks, double *c1, double *z1, double *qd, double **Q, int numVary, int numVarx, int numVarz, int numVarBlock);// Q cost primal
void createDemand(int ClientN, int numBlocks, double **D);// D rhs primal



int main(int argc, char *argv[]) {
  clock_t t0,t1;

  if (argc < 6 ){   //|| argc % 2 != 0) {
    cerr << "usage: SCPrimal need the <number of Supplies>  <number Pfailities> <number of clients> and <number of blocks>  "<< endl
         << "example: ./SCPrimal -SupplyChainEsc #Supply #pFacilities #Clients #scenarios -type_comp_dy 0 \n"
         << endl;

    write_options();
    exit(-1);
  }

 // read some options from command line
 int supplyN = 0; //number of supply nodes
 int Pfacility = 0; //number of the processing facilities
 int ClientN = 0; //number of clients
 int numBlocks = 0;  // number of blocks (scenarios 1ºstage+2ºscenarios)
  int index_outfile= -1;
  int index_mpsfile= -1;
  bool onlymps= false;
  bool quad= false;
  read_options_SCPrimal(argc,argv,index_outfile,index_mpsfile,onlymps,quad,supplyN, Pfacility, ClientN, numBlocks);



cout<<"INICIA"<<endl;
//for(int i = 0; i < argc; i++) 
//      cout << argv[i] << " "; 
//   cout << endl; 

// int supplyN = atoi(argv[1]); //number of supply nodes
// int Pfacility = atoi(argv[2]); //number of the processing facilities
// int ClientN = atoi(argv[3]); //number of clients
// int numBlocks = atoi(argv[4]);  // number of blocks (scenarios 1ºstage+2ºscenarios)

 int numVary=supplyN+Pfacility;  // number of variables y -to build or not-
 int numVarx = supplyN*Pfacility+Pfacility*ClientN; // number of variables x  -arcs- numArcs
 int numVarzS1= ClientN*2; //shorfall of product + slack per ClientN
 int nozeroN1=numVarx+(Pfacility*ClientN)+numVarzS1; //number of non-zeros of 1º Block con demand slacks 
 int nozeroN2x=numVarx+numVary; //number of non-zeros of 1º Block con slacks (Parte 2a +8slacks)  
 int numVarz = numVarzS1 + numVary + numVarx; // number of variables z+ slack(3demanda+8capacidadPyS+U) 
 int numVarBlock= numVary+ (2*numVarx) + numVarz; //with splitting 
 int numConstP1= Pfacility + ClientN; // number of constraints for 1º-block  (Parte 1) 
 int numConstP2 = supplyN + Pfacility + numVarx;  // number of constraints for 1º-blocks 
 int numVarz2= numVarzS1 + numVary; // number of variables z+ slacks(3demanda+8capacidadPyS) 
 int linking = 0; // if 0 then linking are of type SPLITTING; if 1 then linking are GENERAL matrices 
 int Lslacks = (numVary+numVarx)*(numBlocks-1); 
 int numVarxz=numVarx + numVarz;
 int numConst = numConstP1+numConstP2;
 int nozeroN2 = numVary+nozeroN2x+3*numVarx;
 int nozeroN = nozeroN1+nozeroN2;

 int *aN1=NULL, *jN1=NULL, *iN1=NULL;
 int *iN2y=NULL, *jN2y=NULL, *aN2y=NULL;
 int *iN2x=NULL, *jN2x=NULL, *aN2x=NULL;
 double *c1=NULL, *z1=NULL, *qd=NULL;
 double **Q, **D;

  srand(50); 

cout<<"numBlocks= "<<numBlocks<<endl;
cout<<"supplyN= "<<supplyN<<endl;
cout<<"Pfacility= "<<Pfacility<<endl;
cout<<"ClientN= "<<ClientN<<endl;
cout<<"1stStage= "<<numVarx+numVary<<endl;



  iN1=new int[nozeroN1];
  jN1=new int[nozeroN1];
  aN1=new int[nozeroN1];
  iN2y=new int[numVary];
  jN2y=new int[numVary];
  aN2y=new int[numVary];
  iN2x=new int[nozeroN2x];
  jN2x=new int[nozeroN2x];
  aN2x=new int[nozeroN2x];
  c1=new double[numVary];
  z1=new double[ClientN];
  qd=new double[numVarx];

//inicializas Q
Q = new double* [numBlocks];
for(int b=0; b<numBlocks; b++){ 
	Q[b] = new double[numVarBlock]; 
}//for

for(int b=0; b<numBlocks; b++){ 
	for(int auxi=0; auxi<numVarBlock; auxi++){
 		Q[b][auxi] = 0.0; 
	}
}

//inicializas D
D = new double* [numBlocks];
for(int b=0; b<numBlocks; b++){ 
 D[b] = new double[ClientN];
  for(int auxi=0; auxi<ClientN; auxi++){
   D[b][auxi]=0;
  }//for
}//for

  createBalancePfacility(supplyN, Pfacility, ClientN, iN1, jN1, aN1, numVarx);
  createDemandClientN(supplyN, Pfacility, ClientN, iN1, jN1, aN1, numVarx, numVarzS1);
  createCapacitySPy(iN2y, jN2y, aN2y, numVary);
  createCapacitySPxH(supplyN, Pfacility, ClientN, iN2x, jN2x, aN2x, numVarx, nozeroN2x, numVarzS1);
  createInvestmentSP(c1, numVary); 
  createShortageC(ClientN, z1); 
  createTransportationSPyPC(supplyN, Pfacility, qd, numVarx);
  createCost(supplyN, Pfacility, ClientN, numBlocks, c1, z1, qd, Q, numVary, numVarx, numVarz, numVarBlock);
  createDemand(ClientN, numBlocks, D);


  MatrixBlockIP *N =  NULL; 
  N = new MatrixBlockIP[numBlocks];

  int *iW, *jW;
  double *WN; 
  int nz_aux;


for(int b=0; b<numBlocks; b++){ 
N[b].create_general_matrix_format_ija(numConst, numVarBlock, nozeroN, iW, jW, WN); 

		int k = 0, r = 0, s = 0, ss = 0, sss = 0;
		for (int i = 0; i < nozeroN; ++i) {
			if (i<nozeroN1){ 
				iW[i] = iN1[i];
				jW[i] = jN1[i]+ numVary + numVarx; 
				WN[i] = aN1[i];
				//cout<<"i= "<<iW[i]<<"  j= "<<jW[i]<<" - "<<jN1[i]<<" = Wa["<<i<<"]= "<<WN[i]<<endl; 
			}else{             
				if(i<nozeroN1+numVary){ 
					iW[i] = iN2y[k]+ numConstP1;
					jW[i] = jN2y[k];
					WN[i] = aN2y[k];
		//cout<<"i= "<<iW[i]<<" - "<<iN2y[k]<<" j= "<<jW[i]<<" - "<<jN2y[k]<<" = a["<<i<<"]= "<<WN[i]<<endl; 
					k++;
				}else{
					if(i<nozeroN1+numVary+nozeroN2x){ 
						iW[i] = iN2x[r] + numConstP1;
						jW[i] = jN2x[r]+ numVary + numVarx; 
						WN[i] = 1;
		//cout<<"i= "<<iW[i]<<" - "<<iN2x[r]<<" j= "<<jW[i]<<" - "<<jN2x[r]<<" = a["<<i<<"]= "<<WN[i]<<endl; 
						r++;
					}else{
						if(i<nozeroN1+numVary+nozeroN2x+numVarx){ //U's
							iW[i] = s + numConstP1 + numVary;
							jW[i] = s + numVary;
							WN[i] = -1;
		//cout<<"i= "<<iW[i]<<" j= "<<jW[i]<<" = a["<<i<<"]= "<<WN[i]<<endl; 
							s++;
						}else{  
							if(i<nozeroN1+numVary+nozeroN2x+2*numVarx){  // x de U's
							iW[i] = ss + numConstP1 + numVary;
							jW[i] = ss + numVary + numVarx; 
							WN[i] = 1;
		//cout<<"i= "<<iW[i]<<" j= "<<jW[i]<<" = a["<<i<<"]= "<<WN[i]<<endl; 
							ss++;
							}else{ //  U's slacks
								iW[i] = sss + numConstP1 + numVary;
								jW[i] = sss + numVary +(2*numVarx)+numVarz2;//***numVarU
								WN[i] = 1;
		//cout<<"i= "<<iW[i]<<" j= "<<jW[i]<<" = a["<<i<<"]= "<<WN[i]<<endl;
								sss++;
							}
						}
					}
				}
			}    
		}//for 
}//for b


  MatrixBlockIP *L = NULL;
  //create the linking matrices depending on the model chosen and value of parameter linking
  //MatrixBlockIP *L = new MatrixBlockIP[(linking==0) ? 1 : numBlocks];		
  L = new MatrixBlockIP[(linking==0) ? 1 : numBlocks];	

if (linking==0) {
	cout<<"Splitting format"<<endl;
    // linking of type splitting: blocks, range of splitted variables is 0:numVary+numVarU-1
      L->create_splitting_matrix(numBlocks,0,((numVary+numVarx)-1)); //***numVarU
  }
  else {
  	cout<<"No splitting format"<<endl;
    int *iL= NULL, *jL = NULL;
    double *NL= NULL;

	for(int b=0; b<numBlocks; b++){ 
		if(b==0){
	     	L[b].create_general_matrix_format_ija(((numVary+numVarx)*(numBlocks-1)), numVarBlock, (numVary+numVarx), iL, jL, NL); 
			//cout<<"L["<<b<<"]-> m="<<L[b].m<<" n= "<<L[b].n<<" nz= "<<L[b].nz<<endl;
			for (int i = 0; i < (numVary+numVarx); ++i) { 
   				iL[i] = i;
    			jL[i] = i;
				NL[i] = 1; 
				//cout<<"i= "<<iL[i]<<" j= "<<jL[i]<<" = a["<<i<<"]= "<<NL[i]<<endl;
  			 }
		}else{
			if(b<(numBlocks-1)){    
		     	L[b].create_general_matrix_format_ija(((numVary+numVarx)*(numBlocks-1)), numVarBlock, ((numVary+numVarx)*2), iL, jL, NL); 
				//cout<<"L["<<b<<"]-> m="<<L[b].m<<" n= "<<L[b].n<<" nz= "<<L[b].nz<<endl;
				for (int i = 0; i < ((numVary+numVarx)*2); ++i) {  
					if(i<(numVary+numVarx)){  
   						iL[i] = i;
    					jL[i] = i;
						NL[i] = -1;
					}else{
						iL[i] = i;
    					jL[i] = i-(numVary+numVarx);
						NL[i] = 1;
					}
					//cout<<"i= "<<iL[i]<<" j= "<<jL[i]<<" = a["<<i<<"]= "<<NL[i]<<endl;
  				 }//for 		      
			}else{
		   		L[b].create_general_matrix_format_ija(((numVary+numVarx)*(numBlocks-1)), numVarBlock, (numVary+numVarx), iL, jL, NL); 
				//cout<<"Lfinal["<<b<<"]-> m="<<L[b].m<<" n= "<<L[b].n<<" nz= "<<L[b].nz<<endl;
				for (int i = 0; i < (numVary+numVarx); ++i) {   
    				jL[i] = i;
					NL[i] = -1;
					iL[i] = (((numVary+numVarx)*(numBlocks-1))-((numVary+numVarx)-i)); 
					//cout<<"i= "<<iL[i]<<" j= "<<jL[i]<<" = a["<<i<<"]= "<<NL[i]<<endl;
				}
			}//else 
		}//else
	}//for 
  }//else linking==0

 BlockIP bip; // declare BlockIP problem
    
   double *cost=NULL, *qcost=NULL, *ub=NULL, *rhs=NULL;
  
 // creation of BlockIP problem
  bip.create_problem(BlockIP::LINEAR, cost, qcost, NULL, NULL, ub, rhs, numBlocks, true, N, (linking==0), L);//&N(linking==0)

 // fill arrays of data for problem -cost
  int j = 0; // Q[j][jj] -> j blocks
  int jj= 0; 
	for (int i=0; i<(numBlocks*numVarBlock); i++){	
		if(j==0 && i< numVary + numVarx){   
			cost[i] = Q[j][jj];
		}else{
			cost[i] = (Q[j][jj])*(1.0/(double)numBlocks);
		}
		if(i==(numVarBlock*(j+1))-1){ 
			j++;
			jj=-1;
		}
		jj++;	
	}   

//for check
//	for(int i = 0; i < numVarBlock*numBlocks; i++){
//		cout<< "cost["<<i<<"]= "<<cost[i]<<endl;
//	}

 // upper bound for each variable 
int aux_upp1 = 0;//var per block  aux blocks 
int aux_upp2 = 1;//aux blocks 
int aux_upp3 = 0;//for continue with slacks
    for (int i = 0; i < (numVarBlock*numBlocks); ++i) {
		if(aux_upp1 < numVary){ 
			ub[i] = 1;//bip.get_inf();//built or not (if is bigger than1 =>built)
		}else{
			if(aux_upp1<numVarx){
				ub[i] = 1.0e+6;//maximun Capacity for arcs M 
			}else{
				if(aux_upp1 < numVarBlock){  
    				ub[i] = bip.get_inf();
				}
			}//else
		}//else
		if(i==(numVarBlock*(aux_upp2))-1){ 
			aux_upp2++;
			aux_upp1=-1;
		}
		aux_upp1++;
		aux_upp3++;
    }//for

	for (int i = aux_upp3; i < ((numVarBlock*numBlocks)+Lslacks); ++i) {
	   	ub[i] = 1.0e-8;  
	}//for
/*
    for (int i = 0; i < ((numVarBlock*numBlocks)+Lslacks); ++i) {
		if(i < numVary*){ 
			ub[i] = 1;//bip.get_inf();//built or not (if is bigger than1 =>built)
		}else{
			if(i < numVarBlock*numBlocks){  
    			ub[i] = bip.get_inf();
			}else{
	   			ub[i] = 1.0e-8;  
			}//else
		}//else
    }//for
*/

 //for check
//	for(int i = 0; i < (numVarBlock*numBlocks)+Lslacks; i++){
//		cout<< "ub["<<i<<"]= "<<ub[i]<<endl;
//	}

  // rhs of constraints without Lslacks
  int aux_rhs = 0; // D[aux_rhs][aux_rhs2]-->aux_rhs = blocks 
  int aux_rhs2= 0;
  int rhs3= 0; //constraints per block
	for (int i=0; i<(numConst*numBlocks); i++){ 
		rhs[i]=0;
		if(rhs3>=Pfacility && rhs3<(Pfacility + ClientN)){
			rhs[i] = D[aux_rhs][aux_rhs2];
			aux_rhs2++;
		}
		if(i==(numConst*(aux_rhs+1))-1){ 
			aux_rhs++;
			aux_rhs2=0;
			rhs3=-1;
		}
		rhs3++;		
	}  //for
 
  int rhs3a=(numConst*numBlocks);
  for (int i = rhs3a; i < (rhs3a+Lslacks); ++i){
    rhs[i] = 0;
  };

//for check
/*
ofstream FinalMatrix("FinalMatrixPrimal.xls");
FinalMatrix<<"Q[b][i]"<<endl;
for (int b=0; b<numBlocks; b++){
 for (int i = 0; i < numVarBlock; ++i) { 
     FinalMatrix<< Q[b][i] <<"\t";
 }
 FinalMatrix<< endl;
}FinalMatrix<< endl;

FinalMatrix<<"cost"<<endl;
int iAux2 = -1;
for (int b=0; b<numBlocks; b++){
 for (int i = 0; i < numVarBlock; ++i) { 
     FinalMatrix<< cost[++iAux2] <<"\t";
 }FinalMatrix<< endl;
}
FinalMatrix.close();
*/
// t0= clock();
//  write problem in BlockIPformat, read problem (freed and created) and solve 
//  cout << endl << "Write block-MPS format" << endl;
  bip.write_mps("SCPpruebaMPS");
//  cout << endl << "Write block-matlab format" << endl;
//  bip.write_matlab_format("SCPrimal");
//  cout << endl << "Write BlockIP format" << endl;
//  bip.write_BlockIP_format("SCPrimal3a");
//  cout << "Read  BlockIP format" << endl;
//  bip.read_BlockIP_format("FarmerFG.dat");ls

  // set default parameters BIP problem
 // bip.set_optim_gap(1.0e-2);
 // cout<<" optim_gap = 1.0e-2 "<<endl;

 //bip.set_init_pcgtol(1.0e-2);
 //cout<<" init_pcgtol = 1.0e-2 "<<endl;

 //bip.set_red_pcgtol(.2);
 //cout<<" red_pcgtol = .2 "<<endl;

  // bip.set_type_reg(BlockIP::QUAD_REG);
  // bip.set_factor_reg(1.0e-3);


  // change default parameters according to command line values given by user
  read_options_BlockIP(&bip, argc, argv);


 // set some options
  bip.set_show_specrad(true); // compute and show estimation of spectral radius
//  bip.set_type_direction(BlockIP::SECOND_ORDER);// computes predictor-corrector direction with PCG+Cholesky
//  bip.set_type_comp_dy(BlockIP::FULL_CHOL); // solve normal equation with Full Cholesky (not PCG+Cholesky)

//    bip.set_num_threads(24);//number of threads to solve 
// data of the problem
  cout << "BIPnumConst = " << bip.get_km() << endl; //Get the number of constraints without linking constraints
  cout << "BIPnumVaraibles = " << bip.get_kn() << endl;//Get the number of variables without linking constraints
  cout << "BIPnumLinking = " << bip.get_l_link() << endl; //Get the number of linking constraints.
  cout << "numConstOrig = " << bip.get_num_cons() << endl; //Get the number of constraints of the original 
  cout << "numVarOrig = " << bip.get_num_vars() << endl; //Get the number of variables of the original
cout << " optim_gap= " << bip.get_optim_gap() << endl; //Get the optimality gap de BlockIP 
 cout << "init_pcgtol= " << bip.get_init_pcgtol() << endl; //Get  tolerance of the pcg (default in linear problems 1.e-2)
 cout << "red_pcgtol= " << bip.get_red_pcgtol() << endl; //Get if is 1 the tol of pcg don't change in every iteration, number between [0,1] a cada iteración 
cout << "m_pw_prec= " << bip.get_m_pw_prec() << endl;


//solve

  cout << "Solve Problem ** "<<endl; 
 t0= clock();
  bip.minimize(); // solve problem
 t1= clock();

#ifdef OPENMP
    cout << "Elapsed OpenMP wall clock time: " << bip.get_wclock_time() << endl;
#endif
  cout << "Optimization CPU time: " << (double) (t1-t0)/CLOCKS_PER_SEC << " seconds" << endl;
  cout << "Total IPit= " << bip.get_it() << endl;
  cout << "Total CGit= " << bip.get_cgit() << endl;
  cout << "Primal Objetive = " << bip.get_fobj() << endl;
  cout << "Dual objetive = " << bip.get_dobj() << endl;

// t1= clock();


delete[] iN1;
delete[] jN1;
delete[] aN1;
delete[] iN2y;
delete[] jN2y;
delete[] aN2y;
delete[] iN2x;
delete[] jN2x;
delete[] aN2x;
delete[] c1;
delete[] z1;
delete[] qd;
 
delete [] N;
delete [] L;

// delete Q
for(int b=0; b<numBlocks; b++){ 
 delete[] Q[b];
}
 delete[] Q;

// delete D
for(int b=0; b<numBlocks; b++){ 
 delete[] D[b];
}
 delete[] D;

  return 0;
}//main

void createBalancePfacility(int supplyN, int Pfacility, int ClientN, int *iN1, int *jN1, int *aN1, int numVarx){
  int rr=1, aux1i=0, aux1j=0;
	for(int k=0; k<(numVarx);k++){ 
		if(k<(supplyN*Pfacility)){ 
			iN1[k]=aux1i;
			jN1[k]=aux1j;
			aN1[k]=1;
			aux1j=aux1j+Pfacility;
			if(k==((supplyN*rr)-1)){
				aux1i++;
				aux1j=rr;
				rr++;
			}
		}else{
			if(k==(supplyN*Pfacility)){
				aux1i=0;
				rr=1;	
			}
			iN1[k]=aux1i;
			jN1[k]=k;
			aN1[k]=-1;
			if(k==supplyN*Pfacility+(ClientN*rr)-1){
				aux1i++;
				rr++;
			}
		}//else
	}//for
}//void

void createDemandClientN(int supplyN,int Pfacility, int ClientN, int *iN1, int *jN1, int *aN1, int numVarx, int numVarzS1){
    int rr=1, aux1i=Pfacility,  aux2i=Pfacility, aux3i=Pfacility; 
    int aux1j=supplyN*Pfacility, aux2j=numVarx;
	for(int k=numVarx; k<numVarx+(Pfacility*ClientN+numVarzS1);k++){ 
		if(k<numVarx+Pfacility*ClientN){
			iN1[k]=aux1i;
			jN1[k]=aux1j;
			aN1[k]=1;
			aux1j=aux1j+ClientN;
			if(k==(numVarx+(Pfacility*rr)-1)){
				aux1i++;
				aux1j=rr+supplyN*Pfacility;
				rr++;
			}
		}else{
			if(k<numVarx+Pfacility*ClientN+ClientN){
			iN1[k]=aux2i;
			jN1[k]=aux2j;
			aN1[k]=1;
				aux2i++;
				aux2j++;
			}else{
				iN1[k]=aux3i;
				jN1[k]=aux2j;
				aN1[k]=-1;
				aux3i++;
				aux2j++;
			}//esle
		}//else
	}//for
}//void

void createCapacitySPy(int *iN2y, int *jN2y, int *aN2y,  int numVary){
	for(int k=0; k<(numVary);k++){ 
		iN2y[k]=k;
		jN2y[k]=k;
		aN2y[k]=(rand() % 100 + 250)*(-1);
	}
}//void

void createCapacitySPxH(int supplyN, int Pfacility, int ClientN, int *iN2x, int *jN2x, int *aN2x, int numVarx, int nozeroN2x, int numVarzS1){
   int rr=1, r=1, aux1i=0, aux1j=numVarx+numVarzS1, aux2i=supplyN, aux3i=0;
	for(int k=0; k<nozeroN2x;k++){
		aN2x[k]=1; 
		if(k<(numVarx)){ 					
			jN2x[k]=k;
		}else{	
			jN2x[k]=aux1j;
			aux1j++;
		}
		if(k<(supplyN*Pfacility)){
			iN2x[k]=aux1i;
			if(k==((Pfacility*rr)-1)){
				aux1i++;
				rr++;
			}
		}else{
			if(k<numVarx){ 
				iN2x[k]=aux2i;
				if(k==(supplyN*Pfacility)+((ClientN*r)-1)){
					aux2i++;
					r++;
				}
			}else{
				iN2x[k]=aux3i;
				aux3i++;
			}//else
		}//else
	}//for
}//void

void  createInvestmentSP(double *c1, int numVary){
	for(int k=0; k<(numVary);k++){ 
		c1[k]=(rand() % 100 + 250)*1000;
	}   
}//void

void createShortageC(int ClientN, double *z1){
	for(int k=0; k<(ClientN);k++){ 
		z1[k]=(10+rand()%(16-10))*1000;
	}   
}//void

void  createTransportationSPyPC(int supplyN, int Pfacility, double *qd, int numVarx){
  int n1[numVarx];
  double n2[numVarx];
	for(int k=0; k<numVarx;k++){
		n1[k] =(rand()%11); 
		n2[k] =(double)n1[k]/10; 
		if(k<supplyN*Pfacility){ 
			qd[k]=(60+rand()%(211-60))+n2[k];
		}
		else{
			qd[k]=(200+rand()%(701-200))+n2[k];			
		}//else
	} //for
}//void

void createCost(int supplyN, int Pfacility, int ClientN, int numBlocks, double *c1, double *z1, double *qd, double **Q,  int numVary, int numVarx, int numVarz, int numVarBlock){
 int rr = 1; //for bloque 
 for(int b=0; b<numBlocks; b++){ 
 //Unit production cost S U P 
	double Upc[Pfacility] ; //unit production cost for every node Pfacility 
	for (int pp = 0; pp < Pfacility; ++pp ){
		Upc[pp] = (rand() % 250 + 550); 
	}
// for check Upc
//	for (int pp = 0; pp < Pfacility; ++pp ){
//		 cout<<" Upc["<<pp<<"]= "<<Upc[pp]<<endl;
//	}

 double qp[numVarx];
	int kk = 0; // index per Pfacility
	int kkp = 1;//index per SupplyN
	for (int i = 0; i < numVarx; ++i){ 
		if (i < Pfacility*kkp ){
			qp[i]= Upc[kk];
			kk++;
			if(kk == Pfacility){
				kk=0;
				kkp++;
			}
			if(kkp > supplyN){
				kkp=0;
			}
		}else{
			qp[i]=0;
		}
	}//for numVarx
// for check Upc
//	for (int i = 0; i < numVarx; i++ ){
//		 cout<<" qp["<<i<<"]= "<<qp[i]<<endl;
//	}

  //cost per arc
  double c[(numVarBlock)];
  int p = 0;  //index qd
  int z = 0; //index per ClientN penalization
  for (int i = 0 ; i < numVarBlock; ++i ) { 
    if(i<numVary){  
		if (rr==1){ 
			c[i] = c1[i];
		}else{
			c[i] = 0; 
		}
	}else{
		if(i<(numVary + numVarx)){   
			if (rr==1){ 
				c[i]=1000; 
			}else{
				c[i] = 0; 
			}
		}else{
			if(i<(numVary+(2*numVarx))){ 	
				c[i] = qd[p]+qp[p];
				p++;
				if(p ==numVarx) p=0;
			}else{
				if(i<(numVary +(2*numVarx) + numVarz)){  
					if(z<ClientN){
						c[i] = z1[z];
						z++;
					}else{
						c[i]=0; 
					}
					if(z == numVarz) z=0;
				}//if
			}//else
		}//else 
	}//else 
   }//for
 rr++;
 for (int i = 0; i < numVarBlock; ++i) { 
     Q[b][i] = c[i];
     }
 } //for
}//void

void createDemand(int ClientN, int numBlocks, double **D){  // right hand side b -demand per ClientN 
 for(int b=0; b<numBlocks; b++){ 
	for (int i = 0; i < ClientN; ++i) { 
		D[b][i] = (rand() % 250 + 550);
	}
 }
}//void

void write_options() {
  cout <<
  "  -SupplyChainEsc {numberSupplies} {numberPfacilities} {numberClients} {Escenarios}(required) \n"
  "  -out {name}\n"
  "    Output file for optimal variables (if not provided, solution is not reported)\n"
  "  -mps {name}\n"
  "    If name is provided, an mps format of the problem is written to filename 'name'\n"
  "  -only_mps {name}\n"
  "    If name is provided, the program stops after  writing mps format (problem is not solved)\n"
  "  -quad {y/n}\n"
  "    If y costs are considered quadratic; otherwise linear (deefault is linear)\n"
  "  -inf {value}\n"
  "    Set infinity value to be considered\n"
  "  -m_pw_prec {value}\n"
  "    Set number of terms used as preconditioner of the power series expansion of (D-C'*B^{-1}*B)^{-1}\n"
  "  -sigma {value}\n"
  "    Set reduction of the centrality parameter at each IP iteration\n"
  "  -rho {value}\n"
  "    Set reduction of the step-length for the primal and dual variables at each IP iteration\n"
  "  -optim_gap {value}\n"
  "    Set optimality gap tolerance\n"
  "  -optim_pfeas {value}\n"
  "    Set primal feasibility tolerance\n"
  "  -optim_dfeas {value}\n"
  "    Set dual feasibility tolerance\n"
  "  -output_freq {value}\n"
  "    Set output information lines will be printed each output_freq IP iterations\n"
  "  -output {value}\n"
  "    Set type of output. Can take the values: 0= NONE, 1= SCREEN, 2= FILE, 3=BOTH\n"
  "  -maxiter {value}\n"
  "    Set maximum number of IP iterations\n"
  "  -min_pcgtol {value}\n"
  "    Set minimum tolerance for the conjugate gradient\n"
  "  -red_pcgtol {value}\n"
  "    Set reduction of pcg_tol at each IP iteration\n"
  "  -init_pcgtol {value}\n"
  "    Set initial tolerance for the conjugate gradient (by default PCG_TOL_LIN if linear problem, PCG_TOL_QUAD if quadratic or -convex- nonlinear)\n"
  "  -type_start_point {value}\n"
  "    Set how starting point is computed. Can take the values: 0= SIMPLE, 1= QUAD_EQCONS_PROB\n"
  "  -type_comp_dy {value}\n"
  "    Set how dy direction is computed. Can take the values: 0= CHOL_PWRS_PCG, 1= FULL_CHOL, 2= HYBRID_PCG, 3= CHOL_THETA0_PCG, 4= PWRS_THETA0_PCG, 5=THETA0_PWRS_PCG, 6=AUT_PWRSTHETA0_PCG, 7=DYN_PWRSTHETA0_PCG\n"
  "  -type_direction {value}\n"
  "    Set which direction is computed. Can take the values: 0= NEWTON, 1= SECOND_ORDER, 2= AUTOMATIC\n"
  "  -deactivateLnk {value}\n"
  "    Set flag on deactivation of linking\n"
  "  -type_reg {value}\n"
  "    Set type of regularization performed. Can take the values: 0=NO_REG, 1= QUAD_REG, 2= PROX_REG\n" 
  "  -factor_reg {value}\n"
  "    Set initial value for regularization term\n" 
  "  -show_specrad {y/n}\n"
  "    Whether estimation of spectral radius by Ritz values is shown at each IP iteration\n"
  "  -show_princ_angles {y/n}\n"
  "    Whether average principal angles between L and N are shown when computed\n"
  "  -threshold_angle {value}\n"
  "    Set threshold principal angle to switch between PWRS and THETA0 preconditioners\n"
  "  -threshold_specrad {value}\n"
  "    Set threshold spectral radius to switch between PWRS and THETA0 preconditioners\n"
  "  -it_Theta0PWRS {value}\n"
  "    Set number of iterations between checks to switch between PWRS and THETA0\n"
  "  -type_comp_angle {value}\n"
  "    Set how principal angles are computed. Possible values are: 0= LINEARCOMB, 1= AVERAGE\n"
  "  -gap_changeChol {value}\n"
  "    Change to Cholesky if duality gap below this value\n"
  "  -zero_pivots {value}\n"
  "    Max number of zero pivots allowed in Cholesky factorization\n"
  "  -show_zero_pivots {y/n}\n"
  "    Show_zero_pivots at each IP iteration\n"
  "  -stop_if_PCG_fails {y/n}\n"
  "    If PCG fails stop, do not switch to Cholesky, useful if instance is large\n"
  "  -freevars {0/1}\n"
  "    How to deal with free variables: 0=SPLIT , 1=REGULARIZE\n"
  "  -threads n \n"
  "    Set number of threads (by default 1; if n<=1, 1; otherwise min{n,max number of threads})\n"
  << endl;
}//end void write_options()

void read_options_SCPrimal(int argc, char *argv[], int &index_outfile, int &index_mpsfile, bool &onlymps, bool &quad, int &supplyN, int &Pfacility, int &ClientN, int &numBlocks) {
  char filename[512];
  char c;
//  int supplyNa, Pfacilitya, ClientNa, numBloksa;
  for (int argIndex = 1; argIndex < argc; argIndex += 2) {
    if (!strcmp(argv[argIndex], "-SupplyChainEsc") && sscanf(argv[argIndex+1], "%i", &supplyN) && sscanf(argv[argIndex+2], "%i", &Pfacility) &&  sscanf(argv[argIndex+3], "%i", &ClientN) && sscanf(argv[argIndex+4], "%i", &numBlocks) ){
	argIndex +=3;
        index_outfile= argIndex+1;
	}//if !strcmp(argv[argIndex]
    else if (!strcmp(argv[argIndex], "-out") && sscanf(argv[argIndex+1], "%s", &filename) )
      index_outfile= argIndex+1;
    else if (!strcmp(argv[argIndex], "-mps") && sscanf(argv[argIndex+1], "%s", &filename) )
      index_mpsfile= argIndex+1;
    else if (!strcmp(argv[argIndex], "-only_mps") && sscanf(argv[argIndex+1], "%s", &filename) ) {
      index_mpsfile= argIndex+1;
      onlymps= true;
    }
    else if (!strcmp(argv[argIndex], "-quad") && sscanf(argv[argIndex+1], "%c", &c) ) {
      quad= (c== 'y');
    }
    else if (!strcmp(argv[argIndex], "-inf") ||
             !strcmp(argv[argIndex], "-m_pw_prec") ||
             !strcmp(argv[argIndex], "-sigma") ||
             !strcmp(argv[argIndex], "-rho") ||
             !strcmp(argv[argIndex], "-optim_gap") ||
             !strcmp(argv[argIndex], "-optim_pfeas") ||
             !strcmp(argv[argIndex], "-optim_dfeas") ||
             !strcmp(argv[argIndex], "-output_freq") ||
             !strcmp(argv[argIndex], "-output") ||
             !strcmp(argv[argIndex], "-maxiter") ||
             !strcmp(argv[argIndex], "-min_pcgtol") ||
             !strcmp(argv[argIndex], "-red_pcgtol") ||
             !strcmp(argv[argIndex], "-init_pcgtol") ||
             !strcmp(argv[argIndex], "-type_start_point") ||
             !strcmp(argv[argIndex], "-type_comp_dy") ||
             !strcmp(argv[argIndex], "-type_direction") ||
             !strcmp(argv[argIndex], "-deactivateLnk") ||
             !strcmp(argv[argIndex], "-type_reg") ||
             !strcmp(argv[argIndex], "-factor_reg") ||
             !strcmp(argv[argIndex], "-show_specrad") ||
             !strcmp(argv[argIndex], "-show_princ_angles") ||
             !strcmp(argv[argIndex], "-threshold_angle") ||
             !strcmp(argv[argIndex], "-threshold_specrad") ||
             !strcmp(argv[argIndex], "-it_Theta0PWRS") ||
             !strcmp(argv[argIndex], "-type_comp_angle") ||
             !strcmp(argv[argIndex], "-gap_changeChol") ||
             !strcmp(argv[argIndex], "-zero_pivots") ||
             !strcmp(argv[argIndex], "-show_zero_pivots") ||
             !strcmp(argv[argIndex], "-stop_if_PCG_fails") ||
		     !strcmp(argv[argIndex], "-freevars") ||
             !strcmp(argv[argIndex], "-threads") 
             )
      // do nothing 
      {}
    else {
      cerr << "usage: SCPrimal  {options}\n";
      write_options();
      exit(-1);
    }
  }
}
 


void read_options_BlockIP(BlockIP *bip, int argc, char *argv[]) {
  double d;
  int i;
  char c;
  for (int argIndex = 6; argIndex < argc; argIndex += 2) {
    if (!strcmp(argv[argIndex], "-inf") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_inf(d);
    else if (!strcmp(argv[argIndex], "-m_pw_prec") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_m_pw_prec(i);
    else if (!strcmp(argv[argIndex], "-sigma") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_sigma(d);
    else if (!strcmp(argv[argIndex], "-rho") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_rho(d);
    else if (!strcmp(argv[argIndex], "-optim_gap") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_optim_gap(d);
    else if (!strcmp(argv[argIndex], "-optim_pfeas") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_optim_pfeas(d);
    else if (!strcmp(argv[argIndex], "-optim_dfeas") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_optim_dfeas(d);
    else if (!strcmp(argv[argIndex], "-output_freq") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_output_freq(i);
    else if (!strcmp(argv[argIndex], "-output") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_output((BlockIP::OUTPUT)i);
    else if (!strcmp(argv[argIndex], "-maxiter") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_maxiter(i);
    else if (!strcmp(argv[argIndex], "-min_pcgtol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_min_pcgtol(d);
    else if (!strcmp(argv[argIndex], "-red_pcgtol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_red_pcgtol(d);
    else if (!strcmp(argv[argIndex], "-init_pcgtol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_init_pcgtol(d);
    else if (!strcmp(argv[argIndex], "-type_start_point") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_start_point((BlockIP::TYPE_START_POINT)i);
    else if (!strcmp(argv[argIndex], "-type_comp_dy") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_comp_dy((BlockIP::TYPE_COMP_DY)i);
    else if (!strcmp(argv[argIndex], "-type_direction") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_direction((BlockIP::TYPE_DIRECTION)i);
    else if (!strcmp(argv[argIndex], "-deactivateLnk") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_deactivateLnk(i);
    else if (!strcmp(argv[argIndex], "-type_reg") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_reg((BlockIP::TYPE_REG)i);
    else if (!strcmp(argv[argIndex], "-factor_reg") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_factor_reg(d);
    else if (!strcmp(argv[argIndex], "-show_specrad") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_show_specrad(c=='y');
    else if (!strcmp(argv[argIndex], "-show_princ_angles") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_show_princ_angles(c=='y');
    else if (!strcmp(argv[argIndex], "-threshold_angle") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_threshold_angle(d);
    else if (!strcmp(argv[argIndex], "-threshold_specrad") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_threshold_specrad(d);
    else if (!strcmp(argv[argIndex], "-it_Theta0PWRS") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_it_Theta0PWRS(i);
    else if (!strcmp(argv[argIndex], "-type_comp_angle") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_type_comp_angle((BlockIP::TYPE_COMP_ANGLE)i);
    else if (!strcmp(argv[argIndex], "-gap_changeChol") && sscanf(argv[argIndex+1], "%lf", &d))
      bip->set_gap_change_to_Chol(d);
    else if (!strcmp(argv[argIndex], "-zero_pivots") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_allowed_zero_pivots(i);
    else if (!strcmp(argv[argIndex], "-show_zero_pivots") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_show_zero_pivots(c=='y');
    else if (!strcmp(argv[argIndex], "-stop_if_PCG_fails") && sscanf(argv[argIndex+1], "%c", &c))
      bip->set_stop_if_PCG_fails(c=='y');
    else if (!strcmp(argv[argIndex], "-freevars") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_deal_freevars((BlockIP::TYPE_DEAL_FREEVARS)i);
    else if (!strcmp(argv[argIndex], "-threads") && sscanf(argv[argIndex+1], "%i", &i))
      bip->set_num_threads(i);
    else if (!strcmp(argv[argIndex], "-SupplyChainEsc")||
	     !strcmp(argv[argIndex], "-out")||
	     !strcmp(argv[argIndex], "-mps")||
	     !strcmp(argv[argIndex], "-only_mps") ||
	     !strcmp(argv[argIndex], "-quad")
	     )
      // do nothing
      {}
    else {
      cerr << "usage: Bip_BlockIP {options}\n";
      write_options();
      exit(-1);
    }
  }
}//end read_options_BlockIP(BlockIP *bip, int argc, char *argv[])

